
/******************************************************************************
ļ  : Drv_Rtc.c
  : STM32F4RTC
  :
޸  : 2014-08-26      

******************************************************************************/

#include "Drv_SpiFlash.h"
#include "Drv_CommuSPI2.h"


u8 SpiFlashError = 0;

osMutexId SpiFlashMutex = NULL;

/* Private typedef -----------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/
#define sFLASH_CMD_WR_BYTE        0x02  /* Write one byte to Memory instruction */
#define sFLASH_CMD_WR_AAI_WORD    0xAD  /* AAI-Word-Program. Auto Address Increment Programming to Memory instruction */
#define sFLASH_CMD_EBSY           0x70  /* Enable SO to output RY/BY# status during AAI programming */
#define sFLASH_CMD_DBSY           0x80  /* Disable SO to output RY/BY# status during AAI programming */
#define sFLASH_CMD_WRDI           0x04  /* Write-Disable instruction */
#define sFLASH_CMD_EWSR           0x50  /* Enable-Write-Status-Register */


//#define sFLASH_CMD_WRITE          0xAD  /* Write to Memory instruction */

#define sFLASH_CMD_WRSR           0x01  /* Write Status Register instruction */
#define sFLASH_CMD_WREN           0x06  /* Write enable instruction */
#define sFLASH_CMD_READ           0x03  /* Read from Memory instruction */
#define sFLASH_CMD_RDSR           0x05  /* Read Status Register instruction  */
#define sFLASH_CMD_RDID           0x9F  /* Read identification */
#define sFLASH_CMD_SE             0x20  /* Sector Erase instruction */

#define sFLASH_CMD_BE_32K         0x52  /* Block Erase instruction */
#define sFLASH_CMD_BE_64K         0xD8  /* Block Erase instruction */

#define sFLASH_CMD_BE             0x60  /* Bulk Erase instruction */

#define sFLASH_WIP_FLAG           0x01  /* Write In Progress (WIP) flag */

#define sFLASH_DUMMY_BYTE         0xFF
#define sFLASH_SPI_PAGESIZE       0x100
#define sFLASH_CMD_WRITE          0x02


/**
  * @brief  Sends a byte through the SPI interface and return the byte received
  *         from the SPI bus.
  * @param  byte: byte to send.
  * @retval The value of the received byte.
  */
u8 sFLASH_SendByte(u8 byte)
{
  return CommuSPI2_SendByte(byte);
}

/**
  * @brief  Reads a byte from the SPI Flash.
  * @note   This function must be used only if the Start_Read_Sequence function
  *         has been previously called.
  * @param  None
  * @retval Byte Read from the SPI Flash.
  */
u8 sFLASH_ReadByte(void)
{
  return CommuSPI2_ReadByte(sFLASH_DUMMY_BYTE);
}

/**
  * @brief  Sends a Half Word through the SPI interface and return the Half Word
  *         received from the SPI bus.
  * @param  HalfWord: Half Word to send.
  * @retval The value of the received Half Word.
  */
u16 sFLASH_SendHalfWord(u16 HalfWord)
{
  return CommuSPI2_SendHalfWord(HalfWord);
}


/**
  * @brief  Enables the write access to the FLASH.
  * @param  None
  * @retval None
  */
void sFLASH_WriteEnable(void)
{
  /*!< Select the FLASH: Chip Select low */
  sFLASH_CS_HIGH();
  sFLASH_CS_HIGH();
  sFLASH_CS_LOW();

  /*!< Send "Write Enable" instruction */
  sFLASH_SendByte(sFLASH_CMD_WREN);

  /*!< Deselect the FLASH: Chip Select high */
  sFLASH_CS_HIGH();
  sFLASH_CS_HIGH();
}

/**
  * @brief  Initiates a read data byte (READ) sequence from the Flash.
  *   This is done by driving the /CS line low to select the device, then the READ
  *   instruction is transmitted followed by 3 bytes address. This function exit
  *   and keep the /CS line low, so the Flash still being selected. With this
  *   technique the whole content of the Flash is read with a single READ instruction.
  * @param  ReadAddr: FLASH's internal address to read from.
  * @retval None
  */
void sFLASH_StartReadSequence(u32 ReadAddr)
{
  /*!< Select the FLASH: Chip Select low */
  sFLASH_CS_LOW();

  /*!< Send "Read from Memory " instruction */
  sFLASH_SendByte(sFLASH_CMD_READ);

  /*!< Send the 24-bit address of the address to read from -------------------*/
  /*!< Send ReadAddr high nibble address byte */
  sFLASH_SendByte((ReadAddr & 0xFF0000) >> 16);
  /*!< Send ReadAddr medium nibble address byte */
  sFLASH_SendByte((ReadAddr& 0xFF00) >> 8);
  /*!< Send ReadAddr low nibble address byte */
  sFLASH_SendByte(ReadAddr & 0xFF);
}

/**
  * @brief  Polls the status of the Write In Progress (WIP) flag in the FLASH's
  *         status register and loop until write opertaion has completed.
  * @param  None
  * @retval None
  */
void sFLASH_WaitForWriteEnd(void)
#if 0
{
  u8 flashstatus = 0;
  u32 Flag = 0;
  /*!< Loop as long as the memory is busy with a write cycle */
  do
  {

  sFLASH_CS_HIGH();
  sFLASH_CS_HIGH();
  /*!< Select the FLASH: Chip Select low */
  sFLASH_CS_LOW();

  /*!< Send "Read Status Register" instruction */
  sFLASH_SendByte(sFLASH_CMD_RDSR);


    /*!< Send a dummy byte to generate the clock needed by the FLASH
    and put the value of the status register in FLASH_Status variable */
    flashstatus = sFLASH_SendByte(sFLASH_DUMMY_BYTE);
    Flag++;


  /*!< Deselect the FLASH: Chip Select high */
  sFLASH_CS_HIGH();
  sFLASH_CS_HIGH();
  }
  while ((flashstatus & sFLASH_WIP_FLAG) == SET); /* Write in progress */
}
#else
{
  u8 flashstatus = 0;
  u32 Flag = 0;

  sFLASH_CS_HIGH();
  sFLASH_CS_HIGH();
  /*!< Select the FLASH: Chip Select low */
  sFLASH_CS_LOW();

  /*!< Send "Read Status Register" instruction */
  sFLASH_SendByte(sFLASH_CMD_RDSR);

  Flag = 100000;
  /*!< Loop as long as the memory is busy with a write cycle */
  do
  {
    /*!< Send a dummy byte to generate the clock needed by the FLASH
    and put the value of the status register in FLASH_Status variable */
    flashstatus = sFLASH_SendByte(sFLASH_DUMMY_BYTE);
    Flag--;
  }
  while(Flag && ((flashstatus & sFLASH_WIP_FLAG) == SET)); /* Write in progress */

  if(Flag)
    SpiFlashError = 0;
  else
    SpiFlashError = 1;

  /*!< Deselect the FLASH: Chip Select high */
  sFLASH_CS_HIGH();
  sFLASH_CS_HIGH();
}
#endif
/**
  * @brief  Erases the specified FLASH sector.
  * @param  SectorAddr: address of the sector to erase.
  * @retval None
  */
void sFLASH_EraseSector(u32 SectorAddr)
{
  if(osOK != osMutexWait(SpiFlashMutex, 5000))//5
    return;

  /*!< Send write enable instruction */
  sFLASH_WriteEnable();

  /*!< Sector Erase */
  /*!< Select the FLASH: Chip Select low */
  sFLASH_CS_LOW();
  /*!< Send Sector Erase instruction */
  sFLASH_SendByte(sFLASH_CMD_SE);
  /*!< Send SectorAddr high nibble address byte */
  sFLASH_SendByte((SectorAddr & 0xFF0000) >> 16);
  /*!< Send SectorAddr medium nibble address byte */
  sFLASH_SendByte((SectorAddr & 0xFF00) >> 8);
  /*!< Send SectorAddr low nibble address byte */
  sFLASH_SendByte(SectorAddr & 0xFF);
  /*!< Deselect the FLASH: Chip Select high */
  sFLASH_CS_HIGH();
  sFLASH_CS_HIGH();

  /*!< Wait the end of Flash writing */
  sFLASH_WaitForWriteEnd();

 osMutexRelease(SpiFlashMutex);
}

/**
  * @brief  Erases the entire FLASH.
  * @param  None
  * @retval None
  */
void sFLASH_EraseBulk(void)
{
  if(osOK != osMutexWait(SpiFlashMutex, 5000))//5
    return;

  /*!< Send write enable instruction */
  sFLASH_WriteEnable();

  sFLASH_CS_HIGH();
  sFLASH_CS_HIGH();
  /*!< Bulk Erase */
  /*!< Select the FLASH: Chip Select low */
  sFLASH_CS_LOW();
  /*!< Send Bulk Erase instruction  */
  sFLASH_SendByte(sFLASH_CMD_BE);
  /*!< Deselect the FLASH: Chip Select high */
  sFLASH_CS_HIGH();
  sFLASH_CS_HIGH();

  /*!< Wait the end of Flash writing */
  sFLASH_WaitForWriteEnd();

  osMutexRelease(SpiFlashMutex);
}

#if 1
/**
  * @brief  Writes more than one byte to the FLASH with a single WRITE cycle
  *         (Page WRITE sequence).
  * @note   The number of byte can't exceed the FLASH page size.
  * @param  pBuffer: pointer to the buffer  containing the data to be written
  *         to the FLASH.
  * @param  WriteAddr: FLASH's internal address to write to.
  * @param  NumByteToWrite: number of bytes to write to the FLASH, must be equal
  *         or less than "sFLASH_PAGESIZE" value.
  * @retval None
  */
void sFLASH_WritePage(u8* pBuffer, u32 WriteAddr, u16 NumByteToWrite)
{
  /*!< Enable the write access to the FLASH */
  sFLASH_WriteEnable();

  sFLASH_CS_HIGH();
  sFLASH_CS_HIGH();
  /*!< Select the FLASH: Chip Select low */
  sFLASH_CS_LOW();
  /*!< Send "Write to Memory " instruction */
  sFLASH_SendByte(sFLASH_CMD_WRITE);
  /*!< Send WriteAddr high nibble address byte to write to */
  sFLASH_SendByte((WriteAddr & 0xFF0000) >> 16);
  /*!< Send WriteAddr medium nibble address byte to write to */
  sFLASH_SendByte((WriteAddr & 0xFF00) >> 8);
  /*!< Send WriteAddr low nibble address byte to write to */
  sFLASH_SendByte(WriteAddr & 0xFF);

  /*!< while there is data to be written on the FLASH */
  while (NumByteToWrite--)
  {
    /*!< Send the current byte */
    sFLASH_SendByte(*pBuffer);
    /*!< Point on the next byte to be written */
    pBuffer++;
  }

  /*!< Deselect the FLASH: Chip Select high */
  sFLASH_CS_HIGH();
  sFLASH_CS_HIGH();

  /*!< Wait the end of Flash writing */
  sFLASH_WaitForWriteEnd();
}

/**
  * @brief  Writes block of data to the FLASH. In this function, the number of
  *         WRITE cycles are reduced, using Page WRITE sequence.
  * @param  pBuffer: pointer to the buffer  containing the data to be written
  *         to the FLASH.
  * @param  WriteAddr: FLASH's internal address to write to.
  * @param  NumByteToWrite: number of bytes to write to the FLASH.
  * @retval None
  */
void sFLASH_WriteBuffer(u8* pBuffer, u32 WriteAddr, u32 NumByteToWrite)
{
  u8 NumOfPage = 0, NumOfSingle = 0, Addr = 0, count = 0, temp = 0;

  Addr = WriteAddr % sFLASH_SPI_PAGESIZE;
  count = sFLASH_SPI_PAGESIZE - Addr;
  NumOfPage =  NumByteToWrite / sFLASH_SPI_PAGESIZE;
  NumOfSingle = NumByteToWrite % sFLASH_SPI_PAGESIZE;

  if (Addr == 0) /*!< WriteAddr is sFLASH_PAGESIZE aligned  */
  {
    if (NumOfPage == 0) /*!< NumByteToWrite < sFLASH_PAGESIZE */
    {
      sFLASH_WritePage(pBuffer, WriteAddr, NumByteToWrite);
    }
    else /*!< NumByteToWrite > sFLASH_PAGESIZE */
    {
      while (NumOfPage--)
      {
        sFLASH_WritePage(pBuffer, WriteAddr, sFLASH_SPI_PAGESIZE);
        WriteAddr +=  sFLASH_SPI_PAGESIZE;
        pBuffer += sFLASH_SPI_PAGESIZE;
      }

      sFLASH_WritePage(pBuffer, WriteAddr, NumOfSingle);
    }
  }
  else /*!< WriteAddr is not sFLASH_PAGESIZE aligned  */
  {
    if (NumOfPage == 0) /*!< NumByteToWrite < sFLASH_PAGESIZE */
    {
      if (NumOfSingle > count) /*!< (NumByteToWrite + WriteAddr) > sFLASH_PAGESIZE */
      {
        temp = NumOfSingle - count;

        sFLASH_WritePage(pBuffer, WriteAddr, count);
        WriteAddr +=  count;
        pBuffer += count;

        sFLASH_WritePage(pBuffer, WriteAddr, temp);
      }
      else
      {
        sFLASH_WritePage(pBuffer, WriteAddr, NumByteToWrite);
      }
    }
    else /*!< NumByteToWrite > sFLASH_PAGESIZE */
    {
      NumByteToWrite -= count;
      NumOfPage =  NumByteToWrite / sFLASH_SPI_PAGESIZE;
      NumOfSingle = NumByteToWrite % sFLASH_SPI_PAGESIZE;

      sFLASH_WritePage(pBuffer, WriteAddr, count);
      WriteAddr +=  count;
      pBuffer += count;

      while (NumOfPage--)
      {
        sFLASH_WritePage(pBuffer, WriteAddr, sFLASH_SPI_PAGESIZE);
        WriteAddr +=  sFLASH_SPI_PAGESIZE;
        pBuffer += sFLASH_SPI_PAGESIZE;
      }

      if (NumOfSingle != 0)
      {
        sFLASH_WritePage(pBuffer, WriteAddr, NumOfSingle);
      }
    }
  }
}
#endif

/**
  * @brief  Reads FLASH identification.
  * @param  None
  * @retval FLASH identification
  */
u32 sFLASH_ReadID(void)
{
  u32 Temp = 0, Temp0 = 0, Temp1 = 0, Temp2 = 0;

  if(osOK != osMutexWait(SpiFlashMutex, 5000))//5
    return Temp;

  sFLASH_CS_HIGH();
  sFLASH_CS_HIGH();
  /*!< Select the FLASH: Chip Select low */
  sFLASH_CS_LOW();

  /*!< Send "RDID " instruction */
  sFLASH_SendByte(0x9F);

  /*!< Read a byte from the FLASH */
  Temp0 = sFLASH_SendByte(sFLASH_DUMMY_BYTE);

  /*!< Read a byte from the FLASH */
  Temp1 = sFLASH_SendByte(sFLASH_DUMMY_BYTE);

  /*!< Read a byte from the FLASH */
  Temp2 = sFLASH_SendByte(sFLASH_DUMMY_BYTE);

  /*!< Deselect the FLASH: Chip Select high */
  sFLASH_CS_HIGH();
  sFLASH_CS_HIGH();

  Temp = (Temp0 << 16) | (Temp1 << 8) | Temp2;

  osMutexRelease(SpiFlashMutex);

  return Temp;
}

/**
  * @brief  Enables the write access to the FLASH.
  * @param  None
  * @retval None
  */
void sFLASH_WriteRegisterEnable(void)
{
  if(osOK != osMutexWait(SpiFlashMutex, 5000))//5
    return;

  sFLASH_CS_HIGH();
  sFLASH_CS_HIGH();
  /*!< Select the FLASH: Chip Select low */
  sFLASH_CS_LOW();

  /*!< Send "Write Enable" instruction */
  sFLASH_SendByte(sFLASH_CMD_EWSR);

  /*!< Deselect the FLASH: Chip Select high */
  sFLASH_CS_HIGH();
  sFLASH_CS_HIGH();

  osMutexRelease(SpiFlashMutex);
}

/**
  * @brief  Enables the write access to the FLASH.
  * @param  None
  * @retval None
  */
void sFLASH_WriteRegister(u16 Status)
{
  if(osOK != osMutexWait(SpiFlashMutex, 5000))//5
    return;

  sFLASH_CS_HIGH();
  sFLASH_CS_HIGH();
  /*!< Select the FLASH: Chip Select low */
  sFLASH_CS_LOW();

  /*!< Send "Write Enable" instruction */
  sFLASH_SendByte(sFLASH_CMD_WRSR);

  sFLASH_SendByte((Status & 0xFF00)>>8);
  sFLASH_SendByte(Status & 0x00FF);

  /*!< Deselect the FLASH: Chip Select high */
  sFLASH_CS_HIGH();
  sFLASH_CS_HIGH();

  osMutexRelease(SpiFlashMutex);
}

//SST25VF0202B  дһֽ
void sFLASH_WriteOneByte(u8* pBuffer, u32 WriteAddr)
{
  /*!< Enable the write access to the FLASH */
  sFLASH_WriteEnable();

  sFLASH_CS_HIGH();
  sFLASH_CS_HIGH();
  /*!< Select the FLASH: Chip Select low */
  sFLASH_CS_LOW();
  /*!< Send "Write to Memory " instruction */
  sFLASH_SendByte(sFLASH_CMD_WR_BYTE);
  /*!< Send WriteAddr high nibble address byte to write to */
  sFLASH_SendByte((WriteAddr & 0xFF0000) >> 16);
  /*!< Send WriteAddr medium nibble address byte to write to */
  sFLASH_SendByte((WriteAddr & 0xFF00) >> 8);
  /*!< Send WriteAddr low nibble address byte to write to */
  sFLASH_SendByte(WriteAddr & 0xFF);

  /*!< Send the current byte */
  sFLASH_SendByte(*pBuffer);
  /*!< Point on the next byte to be written */

  /*!< Deselect the FLASH: Chip Select high */
  sFLASH_CS_HIGH();
  sFLASH_CS_HIGH();

  /*!< Wait the end of Flash writing */
  sFLASH_WaitForWriteEnd();
}

/**
  * @brief  Enables the write access to the FLASH.
  * @param  None
  * @retval None
  */
void sFLASH_HardBusyEnable(void)
{
  sFLASH_CS_HIGH();
  sFLASH_CS_HIGH();
  /*!< Select the FLASH: Chip Select low */
  sFLASH_CS_LOW();

  /*!< Send "Write Enable" instruction */
  sFLASH_SendByte(sFLASH_CMD_EBSY);

  /*!< Deselect the FLASH: Chip Select high */
  sFLASH_CS_HIGH();
  sFLASH_CS_HIGH();
}

/**
  * @brief  Enables the write access to the FLASH.
  * @param  None
  * @retval None
  */
void sFLASH_HardBusyDisable(void)
{
  sFLASH_CS_HIGH();
  sFLASH_CS_HIGH();
  /*!< Select the FLASH: Chip Select low */
  sFLASH_CS_LOW();

  /*!< Send "Write Enable" instruction */
  sFLASH_SendByte(sFLASH_CMD_DBSY);

  /*!< Deselect the FLASH: Chip Select high */
  sFLASH_CS_HIGH();
  sFLASH_CS_HIGH();
}

/**
  * @brief  Enables the write access to the FLASH.
  * @param  None
  * @retval None
  */
void sFLASH_WriteDisable(void)
{
  sFLASH_CS_HIGH();
  sFLASH_CS_HIGH();
  /*!< Select the FLASH: Chip Select low */
  sFLASH_CS_LOW();

  /*!< Send "Write Enable" instruction */
  sFLASH_SendByte(sFLASH_CMD_WRDI);

  /*!< Deselect the FLASH: Chip Select high */
  sFLASH_CS_HIGH();
  sFLASH_CS_HIGH();
}
#if 0
//SST25VF0202B  дֽ ַҪ16λ Ҫ˫
void sFLASH_WriteMultByte(u8* pBuffer, u32 WriteAddr, u32 NumByteToWrite)
{
  u32 NumHalfWord;

  if(NULL == pBuffer)
    return;
  if(WriteAddr&0x00000001)
    return;//ַҪ16λ
  if(NumByteToWrite < 2)
    return;//Ҫ˫

  NumHalfWord = NumByteToWrite/2;

  /* Enable SO to output RY/BY# status during AAI programming */
  sFLASH_HardBusyEnable();

  /*!< Enable the write access to the FLASH */
  sFLASH_WriteEnable();

  sFLASH_CS_HIGH();
  sFLASH_CS_HIGH();
  /*!< Select the FLASH: Chip Select low */
  sFLASH_CS_LOW();
  /*!< Send "Write to Memory " instruction */
  sFLASH_SendByte(sFLASH_CMD_WR_AAI_WORD);
  /*!< Send WriteAddr high nibble address byte to write to */
  sFLASH_SendByte((WriteAddr & 0xFF0000) >> 16);
  /*!< Send WriteAddr medium nibble address byte to write to */
  sFLASH_SendByte((WriteAddr & 0xFF00) >> 8);
  /*!< Send WriteAddr low nibble address byte to write to */
  sFLASH_SendByte(WriteAddr & 0xFF);

  /*!< Send the current byte */
  sFLASH_SendByte(*pBuffer);
  /*!< Point on the next byte to be written */
  pBuffer++;

  /*!< Send the current byte */
  sFLASH_SendByte(*pBuffer);
  /*!< Point on the next byte to be written */
  pBuffer++;

  /*!< Deselect the FLASH: Chip Select high */
  sFLASH_CS_HIGH();
  sFLASH_CS_HIGH();

  NumHalfWord--;
  sFLASH_CS_LOW();
  while(0x00 == sFLASH_SendByte(sFLASH_DUMMY_BYTE));
  sFLASH_CS_HIGH();
  sFLASH_CS_HIGH();

  /*!< while there is data to be written on the FLASH */
  while(NumHalfWord)
  {
    sFLASH_CS_LOW();
    sFLASH_SendByte(sFLASH_CMD_WR_AAI_WORD);
    sFLASH_SendByte(*pBuffer);
    pBuffer++;
    sFLASH_SendByte(*pBuffer);
    pBuffer++;
    sFLASH_CS_HIGH();
    NumHalfWord--;

    sFLASH_CS_LOW();
    while(0x00 == sFLASH_SendByte(sFLASH_DUMMY_BYTE));
    sFLASH_CS_HIGH();
    sFLASH_CS_HIGH();
  }

  sFLASH_WriteDisable();

  /*!< Wait the end of Flash writing */
  sFLASH_WaitForWriteEnd();

  sFLASH_HardBusyDisable();
}

/**
  * @brief  Writes block of data to the FLASH. In this function, the number of
  *         WRITE cycles are reduced, using Page WRITE sequence.
  * @param  pBuffer: pointer to the buffer  containing the data to be written
  *         to the FLASH.
  * @param  WriteAddr: FLASH's internal address to write to.
  * @param  NumByteToWrite: number of bytes to write to the FLASH.
  * @retval None
  */
void sFLASH_WriteBuffer(u8* pBuffer, u32 WriteAddr, u32 NumByteToWrite)
{
  u32 NumOfPage = 0;
  u8 NumOfSingle = 0;

  if(NULL == SpiFlashMutex)
    return;

  if(osOK != osMutexWait(SpiFlashMutex, 5000))//5
    return;

  if(WriteAddr & 0x00000001)
  {/*ݵַ16λ*/
    sFLASH_WriteOneByte(pBuffer, WriteAddr);
    if(1 == NumByteToWrite)//һֽ
    {
      osMutexRelease(SpiFlashMutex);
      return;
    }

    WriteAddr++;
    pBuffer++;
    NumByteToWrite --;
  }

  NumOfPage =  NumByteToWrite & 0xFFFFFFFE;
  NumOfSingle = NumByteToWrite % 2;

  sFLASH_WriteMultByte(pBuffer, WriteAddr, NumOfPage);
  WriteAddr +=  NumOfPage;
  pBuffer += NumOfPage;

  if(NumOfSingle != 0)/*ݳȷ16λ*/
    sFLASH_WriteOneByte(pBuffer, WriteAddr);

  osMutexRelease(SpiFlashMutex);
}

/**
  * @brief  Reads a block of data from the FLASH.
  * @param  pBuffer: pointer to the buffer that receives the data read from the FLASH.
  * @param  ReadAddr: FLASH's internal address to read from.
  * @param  NumByteToRead: number of bytes to read from the FLASH.
  * @retval None
  */
#endif
void sFLASH_ReadBuffer(u8* pBuffer, u32 ReadAddr, u32 NumByteToRead)
{
  if(NULL == SpiFlashMutex)
    return;

  if(osOK != osMutexWait(SpiFlashMutex, 5000))//5
    return;

  sFLASH_CS_HIGH();
  sFLASH_CS_HIGH();
  /*!< Select the FLASH: Chip Select low */
  sFLASH_CS_LOW();

  /*!< Send "Read from Memory " instruction */
  sFLASH_SendByte(sFLASH_CMD_READ);

  /*!< Send ReadAddr high nibble address byte to read from */
  sFLASH_SendByte((ReadAddr & 0xFF0000) >> 16);
  /*!< Send ReadAddr medium nibble address byte to read from */
  sFLASH_SendByte((ReadAddr& 0xFF00) >> 8);
  /*!< Send ReadAddr low nibble address byte to read from */
  sFLASH_SendByte(ReadAddr & 0xFF);

  while (NumByteToRead--) /*!< while there is data to be read */
  {
    /*!< Read a byte from the FLASH */
    *pBuffer = sFLASH_SendByte(sFLASH_DUMMY_BYTE);
    /*!< Point to the next location where the byte read will be saved */
    pBuffer++;
  }

  /*!< Deselect the FLASH: Chip Select high */
  sFLASH_CS_HIGH();
  sFLASH_CS_HIGH();

 osMutexRelease(SpiFlashMutex);
}

/**
  * @brief  Initializes the peripherals used by the SPI FLASH driver.
  * @param  None
  * @retval None
  */
void sFLASH_Init(void)
{
  SpiFlashMutex = GetCommuMutex();
  if(NULL == SpiFlashMutex)
    SpiFlashError = 1;

  sFLASH_WriteRegisterEnable();
  sFLASH_WriteRegister(0x0000);

  sFLASH_WaitForWriteEnd();
}

